/*
    ucsniff - UCSniff - UC Sniffer
    Copyright (C) 2008 VIPER Lab
    Jason Ostrom <jpo@pobox.com>
    Arjun Sambamoorthy <arjun@sipera.com>
   
    This file is part of UCSniff.

    UCSniff is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    UCSniff is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Modified: Abhijeet Hatekar

*/
#include <tftp_client.h>
#include <targets.h>
#include <ec_dissect.h>
#include <ec_siplog.h>
#include <errno.h>
#include <config.h>
#include <ec_voipglobals.h>
#include <ec_voiplogfiles.h>
#include <ec_liveMonitor.h>
#include <ec_audioDSPPlayer.h>
#include <ec_siprtpmuxer.h>
#include <time.h>

/* Globals */

/* function definitions */

FUNC_DECODER(dissector_sip);
void sip_init(void);
int invite_handler(u_char *, char *, char *, struct packet_object *); 
int ringing_handler(u_char *, char *, char *,struct packet_object *);
int cancel_handler(u_char *, char *, char *, struct packet_object *);
int bye_handler(u_char *, char *, char *, struct packet_object *);
int okay_handler(u_char *, char *, char *, struct packet_object *);
int ack_handler(u_char *, char *, char *, struct packet_object *);
int message_handler(u_char *,char *,char *,struct packet_object *);
int info_handler(u_char *, char *, char *, struct packet_object *);			// added for BOFA demo.
int (*sip_handler)(u_char *, char *, char *, struct packet_object *);
int (*new_decode_method(u_char *)) (u_char *, char *, char *, struct packet_object *);
void create_dissect_string(int,char *,char *);


extern void printtocalllog(sipDB*,int);
extern void printtocalldetaillog(sipDB *);
extern int check_keypadHeader(char *);
extern int getKeypadDigit(char *);

/* Mutex variable for media file muxing */
pthread_mutex_t muxing_mutex = PTHREAD_MUTEX_INITIALIZER;

/* Mutex variable for accessing SIP call structure */
extern pthread_mutex_t sipaccess_mutex;		/* Extern from text/interface */
#define SIPACCESS_LOCK     do{ pthread_mutex_lock(&sipaccess_mutex); } while(0)
#define SIPACCESS_UNLOCK   do{ pthread_mutex_unlock(&sipaccess_mutex); } while(0)

/* Mutex variable for accessing RTP call structure */
pthread_mutex_t rtpaccess_mutex = PTHREAD_MUTEX_INITIALIZER;
#define RTPACCESS_LOCK		do{ pthread_mutex_lock(&rtpaccess_mutex); } while(0)
#define RTPACCESS_UNLOCK	do{ pthread_mutex_unlock(&rtpaccess_mutex); } while(0)

/* Mutex variable for accessing live monitor */
pthread_mutex_t livemonitor_mutex = PTHREAD_MUTEX_INITIALIZER;
#define LIVEMONITOR_LOCK	do{ pthread_mutex_lock(&livemonitor_mutex); } while(0)
#define LIVEMONITOR_UNLOCK	do{ pthread_mutex_unlock(&livemonitor_mutex); } while(0)

/* Adds SIP to the ettercap dissector table */
/* Dissector for sip port 5060, it can be either source port or destination port. 
   Handles both tcp and udp sip sessions */   

void __init sip_init(void)
{
   dissect_add("sip", APP_LAYER_TCP, 5060, dissector_sip);
   dissect_add("sip", APP_LAYER_UDP, 5060, dissector_sip);
}

static const char* ContentLengthStr = "Content-Length";
static const char* EndOfSipMsgHeaderDelimeter = "\r\n\r\n";

static int get_content_length(const u_char* sipMsg)
{
	int contentLength = 0;
	char *cp = strstr((const char*)sipMsg, ContentLengthStr);
	if(cp) {
		while(cp && !isdigit(*cp) )
			++cp;
		contentLength = atoi(cp);
	}
	return contentLength;
}

static int get_actual_length(const u_char* sipMsg, int sipMsgLen)
{
	int actualLength = 0;
	char* cp = strstr((const char*)sipMsg, EndOfSipMsgHeaderDelimeter);
	if(cp) {
		int headerLength = cp - (const char*)sipMsg;
		headerLength += strlen(EndOfSipMsgHeaderDelimeter);
		actualLength = sipMsgLen - headerLength; 
	}
	return actualLength;
}

static char* PartialSipMessage = NULL;

static int have_partial_message()
{
	return (PartialSipMessage != NULL);
}

static void save_partial_message(const u_char* partialSipMsg, int len)
{
	PartialSipMessage = (char*)malloc(len + 1);
	bzero(PartialSipMessage, len + 1);
	memcpy(PartialSipMessage, partialSipMsg, len);
}

static u_char* build_whole_message(const u_char* remainingSipMsg, int* sipMsgLen)
{
	*sipMsgLen += strlen(PartialSipMessage);
	char* new_message = (char*)malloc(*sipMsgLen + 1);
	bzero(new_message, *sipMsgLen + 1);
	strncpy(new_message, PartialSipMessage, *sipMsgLen);
	strncat(new_message, (const char*)remainingSipMsg, *sipMsgLen);
	return (u_char*)new_message;
}

static void clear_partial_message(const u_char* sipMsgToClear)
{
	//free(sipMsgToClear);
	PartialSipMessage = NULL;
}

/* function decoder for sip messages */

FUNC_DECODER(dissector_sip){


	DECLARE_DISP_PTR_END(ptr, end);
	char temp[MAX_ASCII_ADDR_LEN];		/* Hold the Source IP address of the received packet */
	char temp1[MAX_ASCII_ADDR_LEN];		/* Holds the destination IP address of the received packet */

	int bytes = end - ptr;
	/* Skipping if the number of bytes is zero */
	if(bytes == 0)
		return NULL;

	int contentLength;

	if(have_partial_message()) {
		ptr = build_whole_message(ptr, &bytes);
	}
	else if((contentLength = get_content_length(ptr)) > 0) {
		int actualLength = get_actual_length(ptr, bytes);
		if(actualLength < contentLength) {
			save_partial_message(ptr, bytes);
			return NULL;			
		}
	}

	bzero(temp,sizeof(temp));
	bzero(temp1,sizeof(temp1));
	ip_addr_ntoa(&PACKET->L3.src, temp);
	ip_addr_ntoa(&PACKET->L3.dst,temp1);

	SIPLOG("******************************************************************\n");
	SIPLOG("Logging New message received, Source IP %s and Destination IP %s \n",temp,temp1);
	SIPLOG("Source Port = %d and Destination Port = %d \n", ntohs(PACKET->L4.src), ntohs(PACKET->L4.dst)); 
	SIPLOG("------------------------------------------------------------------\n");
	char* printBuf = (char*)malloc(bytes + 1);
	bzero(printBuf, bytes + 1);
	strncpy(printBuf, (const char*)ptr, bytes);
	SIPLOG("%s", printBuf);
	free(printBuf);
	SIPLOG("------------------------------------------------------------------\n");

	/* Initializes the sip uri structure, because this structure holds the temporary values parsed from the sip message */
	initialize_uri_struct();
	
	/* Dump the message received into a pcap file */
	 if(sipdumper != NULL)
                pcap_dump((u_char *)sipdumper,dump_pkthdr,PACKET->packet);

	//SIPLOG("Packet dumped in sipdump.pcap\n");

	/* A simple function pointer, returns which function to be called based on the sip message */  		
	sip_handler = new_decode_method(ptr);
	if(sip_handler) {
		(*sip_handler) (ptr,temp,temp1,PACKET);
		if(have_partial_message()) {
			clear_partial_message(ptr);
		}
	}
	SIPLOG("******************************************************************\n");
	return NULL;
}

int invite_handler(u_char *ptr, char *temp, char *temp1, struct packet_object *packet){

	SIPLOG("INVITE message handler activated\n");

	/* Parses the sip header for source extension, destination extension, call id, cseq */
	int return_decode = decode_sip_header(ptr);
	if(return_decode < 0){
		//SIPLOG("Failed: parsing sip headers\n");
		return -1;
	}

	if(strcmp(uri_obj.cseq_method,"INVITE") != 0){
		//SIPLOG("Failed: CSeq method is not INVITE\n");
                return -1;
	}

	/* Check for from-tag */
	if(strlen(uri_obj.from_tag) < 0) {
		return -1;
	}
	
	char buffer[10] = { '\0' };
	sipDB *currentSipCall = matchsipDBInvite(buffer,sizeof(buffer));
	if((!strcmp(buffer,"server")) && (currentSipCall != NULL)){
		/* Server or UAS has sent a re-Invite */
		SIPLOG("Checking reinvite sent by server\n");
		int ret = check_reinvite_server(currentSipCall,ptr);
		return ret;
	}
	else if((!strcmp(buffer,"client")) && (currentSipCall != NULL)){
		/* Client or UAC has sent a re-Invite */
		SIPLOG("Checking reinvite sent by client\n");
		int ret = check_reinvite_client(currentSipCall,ptr);
		return ret;
	}
	else if((!strcmp(buffer,"newcall")) && (currentSipCall == NULL)){

		/* Check for from extension in monitor mode */
		if(monitor_yes && fromExtension){

			if(strcmp(uri_obj.src_extension,fromExtensionMonitor) != 0){
				return -1;
			}
		}

		//SIPLOG("A new SIP call\n");
		/* when call forwarding is enabled on Cisco IP phones they send invite to TO uri <x-cisco-serviceuri-cfwdall>, skipping this message */
		if((strstr(uri_obj.dst_extension,"x-cisco-serviceuri")) != NULL){
			SIPLOG("Call forwarding enabled on Cisco IP phones, to-uri is x-cisco-serviceuri\n");
			return -1; 
		}

		/* Some Cisco phones use kpml tags to send the dialed party number, we get the dialed number from Remote-Party header in 200 OK or 180 */
       	decode_kpml(ptr);
		
		/* Parses the User-Agent header */
		decode_ua(ptr);

		/* Some sip invite message doesn't have sdp message body */		
		int return_media = decode_media_port(ptr);
		if(return_media < 0){
			SIPLOG("Failed: parsing decode_media_port\n");
			uri_obj.media_src_port = 0;
		}

		int return_vport = decode_video_port(ptr);
		if(return_vport < 0){
			SIPLOG("Failed: parsing decode_video_port\n");
			uri_obj.video_port = 0;
		}

		SIPACCESS_LOCK;
		/* Create, initialize and copies values for the new sip call */
		//if(return_media == 0 && return_vport == 0) {
			currentSipCall = createNewSipCall(temp, temp1, packet, ptr);
			if(currentSipCall == NULL){
				SIPACCESS_UNLOCK;
				return -1;
			}
			printtoSipLog(currentSipCall,NULL,"INVITE");
		//}
		SIPACCESS_UNLOCK;

	}
	return 0;
}

int ringing_handler(u_char *ptr, char *temp, char *temp1, struct packet_object *packet){

	//SIPLOG("180 Ringing message handler activated\n");
	return 0;
}

int cancel_handler(u_char *ptr, char *temp, char *temp1, struct packet_object *packet){

	SIPLOG("Cancel message handler activated \n");
 	int return_decode = decode_sip_header(ptr);
	if(return_decode < 0){
		//SIPLOG("Failed: parsing decode_sip_header\n");
		return -1;
	}

	if(strcmp(uri_obj.cseq_method,"CANCEL") != 0){
		//SIPLOG("Cseq method is not cancel\n");
		return -1;
	}

	/* A cancel request can only be made by the calling party */
	sipDB *currentSipCall = match_cancel_request();
	if(currentSipCall == NULL){
		SIPLOG("Failed: Matching the cancel request with the active current sip calls\n");
		return -1;
	}

	printtoSipLog(currentSipCall,NULL,"CANCEL");
	/* Delete the sip call structure from the call list */
	SIPACCESS_LOCK;
   
	deleteSipCall(currentSipCall);

	SIPACCESS_UNLOCK;

	return 0;
}				

int bye_handler(u_char *ptr, char *temp, char *temp1, struct packet_object *packet){

	rtpSIP* currentRTPCall = NULL;

	SIPLOG("Bye message handler activated\n");
	int return_decode = decode_sip_header(ptr);
	if(return_decode < 0){
		SIPLOG("Failed: parsing decode_sip_header\n");
		return -1;
	}
	
	if(strlen(uri_obj.from_tag) < 0)
	{
		return -1;
	}

	/* Check if to tag is present */
	if(strlen(uri_obj.to_tag) < 0) {
		return -1;
	}
	
	if(strcmp(uri_obj.cseq_method,"BYE") != 0){
		SIPLOG("Failed: CSeq method not matched BYE\n");
		return -1;
	}

    sipDB* currentSipCall = match_bye_request();
    if(currentSipCall == NULL){
		SIPLOG("Failed: Matching the bye request with the current active sip calls\n");
		return -1;
	}

	/* Delete the RTP dissectors for this call */
	deleteRTPDissector(currentSipCall->media_src_port,currentSipCall->media_src_ip);
	if(currentSipCall->video_active == 1){
		deleteRTPDissector(currentSipCall->video_src_port,currentSipCall->media_src_ip);
	}
		
	currentRTPCall = currentSipCall->myRTPCall;
	
	if(currentRTPCall == NULL){
		SIPLOG("Bye, currentRTPCall is NULL returning\n");
        return -1;
   	}

	/* Close the RTP and SIP packet capture files */
    pcap_dump_close(currentRTPCall->dumper);
    pcap_close(currentRTPCall->dp);

#ifndef OS_WINDOWS

	LIVEMONITOR_LOCK;

    // If this call is monitored live, stop it
    if(currentRTPCall->liveMonitor == MONITOR_START){

        stopCurrentlyMonitoringStream();
    }

	LIVEMONITOR_UNLOCK;

#endif

	/* Creates a new thread for muxing reverse and fwd direction audio and for muxing audio and video */
    struct muxVariable{
	    rtpSIP *rtpPtr;
	    int video_active;
    }muxV;

    muxV.rtpPtr = currentRTPCall;
    muxV.video_active = currentSipCall->video_active;

    ec_thread_new("mediaMUX", "mux audio and video streams", &muxMediaFiles, &muxV);
    /* Loop until the media file names are copied from the RTP call structure */
    while(1){

	    //MUXING_LOCK;
	    pthread_mutex_lock(&muxing_mutex);
	    if(currentRTPCall->copiedFiles){
		    //MUXING_UNLOCK;	
		    pthread_mutex_unlock(&muxing_mutex);
		    break;
	    }
	    pthread_mutex_unlock(&muxing_mutex);
	    //MUXING_UNLOCK;
    }
    
    gettimeofday(&currentSipCall->finish,0);
    get_time_string(1,currentSipCall->end_time_string);

    printtocalllog(currentSipCall,2);
    printtocalldetaillog(currentSipCall);
	printtoSipLog(currentSipCall,currentRTPCall,"BYE");

	/* Delete the RTP and SIP structure from the call list */
	if(currentRTPCall != NULL){

		RTPACCESS_LOCK;

		deleteRTPCall(currentRTPCall);

		RTPACCESS_UNLOCK;
	}
	if(currentSipCall != NULL){

		SIPACCESS_LOCK;

		deleteSipCall(currentSipCall);

		SIPACCESS_UNLOCK;
	}
    return 0;
}

int okay_handler(u_char *ptr, char *temp, char *temp1, struct packet_object *packet){

	SIPLOG("200 OK message handler activated\n");

	int return_decode = decode_sip_header(ptr);
	if(return_decode < 0){
		//SIPLOG("Failed: parsing decode_sip_header\n");
		return -1;
	}

	if(strlen(uri_obj.from_tag) < 0)
		return -1;

	/* Check if to tag is present */
	if(strlen(uri_obj.to_tag) < 0)
		return -1;

	/* Handles 200 OK only for INVITE */
	if(strcmp(uri_obj.cseq_method,"INVITE") != 0){
		//SIPLOG("Cseq_method not matching INVITE \n");
		return -1;
	}

	char buffer[10] = { '\0' };
	sipDB* currentSipCall = matchsipOkay(buffer,sizeof(buffer));
	
	if(currentSipCall == NULL){
		//SIPLOG("200 OK message does not match any current active sip calls \n");
		return -1;
	}
	else if((!strcmp(buffer,"server")) && (currentSipCall != NULL)){
		SIPLOG("Server reinvite 200 OK message\n");
		int ret = check_serverReInviteOkay(currentSipCall,ptr);
		return ret;
	}
	else if((!strcmp(buffer,"client")) && (currentSipCall != NULL)){
		SIPLOG("Client reinvite 200 OK message\n");
		int ret = check_clientReInviteOkay(currentSipCall,ptr);
		return ret;
	}
	else{
		if(currentSipCall->okay_active == 1){
			SIPLOG("Probably a duplicate okay message, 200 OK already received\n");
			return -1;
		}

		/* Save the to tag to the SIP call structure */
		memcpy(currentSipCall->to_tag,uri_obj.to_tag,sizeof(currentSipCall->to_tag));

		/* Check if the dst number sent in Invite and the one in 200 OK are same. For proxies re-write the dst extension for outbound calls via SIP Trunking */
		if(strcmp(currentSipCall->dst_extension,uri_obj.dst_extension) != 0 ){
			memset(currentSipCall->dst_extension,'\0',sizeof(currentSipCall->dst_extension));
			strncpy(currentSipCall->dst_extension,uri_obj.dst_extension,sizeof(currentSipCall->dst_extension));
		}

		if(target_yes == 1){
			/* Target mode, checks the targets */ 
			int return_target = okay_targethandler(currentSipCall,ptr);
			if(return_target != 1)
				return -1;
		}
	
		/* The media IP address(connection information) from the SDP message body */
		int return_ip = decode_dialed_ip(ptr);
		if(return_ip < 0){
			//SIPLOG("Failed: parsing decode_dialed_ip\n");
			return -1;
		}

		/* The media port from the SDP message body */
		int return_port = decode_media_port(ptr);
		if(return_port < 0){
			//SIPLOG("Failed: parsing decode_media_port\n");
			return -1;
		}

		/* When a cisco phone gets a call from different vlan, the to-uri will be the contact uri, which is a random number of hex values */
		int return_remote = decode_remote_party(ptr);
		if(return_remote == 0){
			if((strcmp(currentSipCall->dst_extension,uri_obj.dst_extension) != 0) && (strcmp(currentSipCall->src_extension,uri_obj.dst_extension)!= 0)){
				updateRemoteExtension(currentSipCall,uri_obj.dst_extension);
			}
		}
			
		memcpy(currentSipCall->media_dst_ip,uri_obj.ip_addr,MAX_ASCII_ADDR_LEN);
		currentSipCall->media_dst_port = uri_obj.media_src_port;

		/* Checks to see if video is negotiated */

        if(currentSipCall->media_src_port != 0){

			char *rtpmap = NULL;
			char audioCodecType;
			char audioPayloadType;
			int g726SampleSize = 0;
			/* Gets the negotiated audio codec */
			int return_codec = getNegotiatedAudioCodecTypeAndRTPMAP(ptr,&rtpmap,&audioCodecType,&audioPayloadType, &g726SampleSize);
			if(return_codec < 0){
					//INSTANT_USER_MSG("Audio codec not supported by UCSniff, not recording the conversation\n");
			}
			else{
				copyAudioCodecParams(currentSipCall, rtpmap, audioCodecType, audioPayloadType,g726SampleSize);
			}

            int return_vport = decode_video_port(ptr);
            if(return_vport != -1 && currentSipCall->video_src_port != 0){
            	if(uri_obj.video_port != 0){
                	currentSipCall->video_active = 1;
                    currentSipCall->video_dst_port = uri_obj.video_port;

					char *vrtpmap = NULL;
					char *vfmtp = NULL;
					char videoCodecType;
					char videoPayloadType;

					int return_vcodec = getNegotiatedVideoCodecTypeAndRTPMAP(ptr,&vrtpmap,&vfmtp,&videoCodecType,&videoPayloadType);
					if(return_vcodec < 0){
						INSTANT_USER_MSG("Video codec not supported by UCSniff not recording video conversation \n");
						currentSipCall->video_active = 0;
						currentSipCall->video_dst_port = 0;
					}
					else{
						copyVideoCodecParams(currentSipCall, vrtpmap,vfmtp,videoCodecType,videoPayloadType);
					}
                }
                else{
                	currentSipCall->video_active = 0;
                    currentSipCall->video_dst_port = uri_obj.video_port;
                }
           	} 
          	else{
            	currentSipCall->video_active = 0;
            }
		}
        else{
        	int return_vport = decode_video_port(ptr);
            if(return_vport < 0){
            	currentSipCall->video_active = 0;
            }
            else{
            	if(uri_obj.video_port == 0){
                	currentSipCall->video_active = 0;
                    currentSipCall->video_dst_port = uri_obj.video_port;
                }
                else{
                	currentSipCall->video_active = -1;
                    currentSipCall->video_dst_port = uri_obj.video_port;
                }
            }
        }
		printtoSipLog(currentSipCall,NULL,"200 OK");
		currentSipCall->okay_active = 1;
	}

	return 0;
}

int ack_handler(u_char *ptr, char *temp, char *temp1, struct packet_object *packet){

	rtpSIP* currentRTPCall = NULL;

	SIPLOG("ACK message handler activated\n"); 
	int return_decode = decode_sip_header(ptr);
	if(return_decode < 0){
		//SIPLOG("Failed: parsing decode_sip_header\n");
		return -1;
	}

	if(strlen(uri_obj.from_tag) < 0)
		return -1;

	if(strlen(uri_obj.to_tag) < 0)
		return -1;

	if(strcmp(uri_obj.cseq_method,"ACK") != 0){
		//SIPLOG("ACK,cseq method not matching ACK\n");
		return -1;
	}

	char buffer[10] = {'\0'};
	sipDB* currentSipCall = matchsipAck(buffer,sizeof(buffer));
	if(currentSipCall == NULL){
		//SIPLOG("ACK message not matching any of the current active sip calls\n");
		return -1;
	}
	else if((!strcmp(buffer,"client")) && (currentSipCall != NULL)){
		SIPLOG("Client reInvite ack message\n");
		int ret = check_clientReInviteAck(currentSipCall,ptr);
		return ret;
	}
	else if((!strcmp(buffer,"server")) && (currentSipCall != NULL)){
		SIPLOG("Server reInvite ack message\n");
		int ret = check_serverReInviteAck(currentSipCall,ptr);
		return ret;
	}
	else{

		if(currentSipCall->okay_active != 1){

			/* We handle the ACK that is sent for 200 OK ( successful response), not for any other 3xx or 4xx responses */
			SIPLOG("SIP Call(%d) 4xx response received, ACK not received for 200 Ok. Deleting the call. \n",currentSipCall->call_number);

			SIPACCESS_LOCK;

			deleteSipCall(currentSipCall);

			SIPACCESS_UNLOCK;

			return -1;
		}

		if(currentSipCall->ack_active == 1){
			SIPLOG("Probably a duplicate ack received\n");
                        return -1;
		}

		/* If there is no SDP message in INVITE, then there has to be a SDP message in ACK */
        if(currentSipCall->media_src_port == 0){

        	int return_ip = decode_dialed_ip(ptr);
            if(return_ip < 0){
				//SIPLOG("Failed: parsing decode_dialed_ip\n");
                return -1;
			}

            int return_port = decode_media_port(ptr);
            if(return_port < 0){
				//SIPLOG("Failed: parsing decode_media_port\n");
                return -1;
			}
							
			char *rtpmap = NULL;
			char audioCodecType;
			char audioPayloadType;
			int g726SampleSize = 0;

			int return_codec = getNegotiatedAudioCodecTypeAndRTPMAP(ptr,&rtpmap,&audioCodecType,&audioPayloadType,&g726SampleSize);
			if(return_codec < 0){
				//INSTANT_USER_MSG("Audio codec not supported, not recording the conversation\n");
			}
			else{
				copyAudioCodecParams(currentSipCall, rtpmap, audioCodecType, audioPayloadType, g726SampleSize);
			}
            memcpy(currentSipCall->media_src_ip,uri_obj.ip_addr,MAX_ASCII_ADDR_LEN);
            currentSipCall->media_src_port = uri_obj.media_src_port;
		}
        if(currentSipCall->video_active == -1){

        	int return_vport = decode_video_port(ptr);
            if(return_vport < 0){
				currentSipCall->video_active = 0;
 	        }
        	else{
            	if(uri_obj.video_port != 0){
                   	currentSipCall->video_src_port = uri_obj.video_port;
                    currentSipCall->video_active = 1;
				
					char *vrtpmap = NULL;
					char *vfmtp = NULL;
					char videoCodecType;
					char videoPayloadType;

					int return_vcodec = getNegotiatedVideoCodecTypeAndRTPMAP(ptr,&vrtpmap,&vfmtp,&videoCodecType,&videoPayloadType);
					if(return_vcodec < 0){
						INSTANT_USER_MSG("Video codec not support by UCSniff not recording video conversation\n");
						currentSipCall->video_active = 0;
						currentSipCall->video_src_port = 0;
					}
					else{
						copyVideoCodecParams(currentSipCall, vrtpmap,vfmtp,videoCodecType,videoPayloadType);
					}
                }
                else
                	currentSipCall->video_active = 0;
			}
		}

		RTPACCESS_LOCK;

		/* Creates a rtp call structure */
        currentRTPCall = createNewRTPCall(currentSipCall,currentSipCall->media_src_port,currentSipCall->media_src_ip);
		if(currentRTPCall == NULL){
			RTPACCESS_UNLOCK;
			return -1;
		}

		RTPACCESS_UNLOCK;

		/* Add the RTP Dissector for audio */
		addRTPDissector(currentSipCall->media_src_port,currentSipCall->media_src_ip);

		/* Add RTP dissector for video and create the video files */
		if(currentSipCall->video_active == 1){
    	    populatevideoRTPCall(currentRTPCall,currentSipCall->video_src_port,currentSipCall->videoRTPMAP,currentSipCall->videoFMTP,currentSipCall->videoCodecType,currentSipCall->videoPayloadType);
		 	addRTPDissector(currentSipCall->video_src_port,currentSipCall->media_src_ip);
		}
		
		currentSipCall->myRTPCall = currentRTPCall;
		currentSipCall->ack_active = 1;
		currentSipCall->session_active = 1;					

		sip_targetlookup(currentSipCall);

	if(currentSipCall->remoteParty == 1){
		SIPLOG("SIP Call in progress. %s (extension %s, ip %s) calling %s (extension %s, ip %s)\n", currentSipCall->src_username,currentSipCall->src_extension,currentSipCall->media_src_ip,currentSipCall->dst_username,currentSipCall->remote_extension,currentSipCall->media_dst_ip);
        	INSTANT_USER_MSG("SIP Call in progress. %s (extension %s, ip %s) calling %s (extension %s, ip %s)\n", currentSipCall->src_username,currentSipCall->src_extension,currentSipCall->media_src_ip,currentSipCall->dst_username,currentSipCall->remote_extension,currentSipCall->media_dst_ip);
	}
	else{
		SIPLOG("SIP Call in progress. %s (extension %s, ip %s) calling %s (extension %s, ip %s)\n", currentSipCall->src_username,currentSipCall->src_extension,currentSipCall->media_src_ip,currentSipCall->dst_username,currentSipCall->dst_extension,currentSipCall->media_dst_ip);
        	INSTANT_USER_MSG("SIP Call in progress. %s (extension %s, ip %s) calling %s (extension %s, ip %s)\n", currentSipCall->src_username,currentSipCall->src_extension,currentSipCall->media_src_ip,currentSipCall->dst_username,currentSipCall->dst_extension,currentSipCall->media_dst_ip);
	}
    printtocalllog(currentSipCall,1);
	printtoSipLog(currentSipCall,currentRTPCall,"ACK");

	}
	return 0;
}

int message_handler(u_char *ptr, char *temp, char *temp1, struct packet_object *packet){

	int return_from = decode_from_uri(ptr);
	if(return_from < 0)
		return -1;

	int return_to = decode_to_uri(ptr);
	if(return_to < 0)
		return -1;

	int return_check = check_OCSContentType(ptr);
	if(return_check < 0)
		return -1;

	int return_text = parseOCSIMText(ptr);
	if(return_text < 0)
		return -1;

	INSTANT_USER_MSG("IM Message: %s to %s : %s \n", uri_obj.src_extension, uri_obj.dst_extension, uri_obj.ocsIM);
	fprintf(sipfile,"IM Message: %s to %s : %s\n", uri_obj.src_extension, uri_obj.dst_extension, uri_obj.ocsIM);

	free(uri_obj.ocsIM);
	return 0;
}
int info_handler(u_char *ptr, char *temp, char *temp1, struct packet_object *packet)			// added for BOFA demo.
{
	time_t t1;
	struct tm *tm;
	char *timestamp = NULL;
	static int keyCount = 0;

	int return_from = decode_from_uri(ptr);
	if(return_from < 0)
		return -1;

	int return_to = decode_to_uri(ptr);
	if(return_to < 0)
		return -1;

	int return_check = check_keypadHeader((char *)ptr);
	if(return_check < 0)
		return -1;
	
	time(&t1);				// Gets raw time 
	tm = localtime(&t1);			// Gets local time in tm structured format

	timestamp = asctime(tm);		// Print time from tm structure in readable string.

	int return_text = getKeypadDigit((char *)ptr);
	if(return_text < 0)
		return -1;

	if(keyCount == 0){
		INSTANT_USER_MSG("Account number pressed on: %s by: %s to %s : %s",timestamp, uri_obj.src_extension, uri_obj.dst_extension, uri_obj.keypadDigit);
		keyCount += 1;
	}
	else if(!strcmp((const char*)uri_obj.keypadDigit,"#")){
		INSTANT_USER_MSG("%s\n",uri_obj.keypadDigit);
		keyCount = 0;
	}
	else{
		INSTANT_USER_MSG("%s",uri_obj.keypadDigit);
	}

	fprintf(sipfile,"Keypad button Pressed on: %s by: %s to %s : %s \n", timestamp, uri_obj.src_extension,uri_obj.dst_extension,uri_obj.keypadDigit);
	fflush(sipfile);

	free(uri_obj.keypadDigit);
	return 0;
}

int (*new_decode_method(u_char *msg)) (u_char *,char *,char *,struct packet_object *){

	char *result;
    char *string;
    if((result = strstr((char *)msg,"\r\n")) != NULL){

    	int length = result - (char *)msg;
        string = (char *)malloc((length+1) * sizeof(char));
		memset(string,'\0',length+1);
        strncpy(string,(char *)msg,length);
    }
    else{
		SIPLOG("Failed: Parsing the sip request line\n");
        return NULL;
    }
	if(((result = stristr(string,"200 OK")) != NULL)){
        
		free(string);
	    return &okay_handler;
    }
    else if((result=stristr(string,"INVITE")) != NULL){

		free(string);
        return &invite_handler;
    }
	else if((result = stristr(string,"ACK")) != NULL){
        	
		free(string);
        return &ack_handler;
    }
	else if((result = stristr(string,"BYE")) != NULL){
		free(string);
	    return &bye_handler;
    }
	else if((result = stristr(string,"180 Ringing")) != NULL){
			
		free(string);
        return &ringing_handler;
    }
    else if((result = stristr(string,"CANCEL")) != NULL){
        	
		free(string);
	    return &cancel_handler;
    }
	else if((result = stristr(string,"MESSAGE")) != NULL){
		
		free(string);
		return &message_handler;
	}
	else if ((result = stristr(string,"INFO")) != NULL){
		free(string);
		return &info_handler;
	}
    else{
		SIPLOG("SIP Message not supported\n");
		free(string);
        return NULL;
    }
}	

void get_time_string(int mode24,char *tmp)
{
  static char time_str[12] = {0};   /* Stroes the time as a string */
  struct tm *now = NULL;
  int hour = 0;
  time_t time_value = 0;

  time_value = time(NULL);          /* Get time value              */
  now = localtime(&time_value);     /* Get time and date structure */
  hour = now->tm_hour;              /* Save the hour value         */
  if(!mode24 && hour>12)            /* Adjust if 12 hour format    */
    hour -= 12;

#ifdef OS_WINDOWS  
  sprintf(time_str,"%d-%d-%d",hour,now->tm_min,now->tm_sec);
#else
  sprintf(time_str,"%d-%d-%d",hour,now->tm_min,now->tm_sec);
#endif 
  
  strncpy(tmp,time_str,sizeof(time_str));
}

void create_dissect_string(int media_port, char *ip, char *dissect_string){

        char *dissect_name = NULL;
		if((dissect_name = calloc(6 + 3 + MAX_ASCII_ADDR_LEN,sizeof(char))) == NULL){
			INSTANT_USER_MSG("Error: allocating memory for the rtp dissector string \n");
			return;
		}
        char string_port[8];
        strcpy(dissect_name,"rtp");
        snprintf(string_port,sizeof(string_port),"%d",media_port);
        strcat(dissect_name,string_port);
        strcat(dissect_name,ip);
        strncpy(dissect_string,dissect_name,(6+2+MAX_ASCII_ADDR_LEN));
        SIPLOG("Dissector string created is %s\n",dissect_string);
		free(dissect_name);
}

void updateRemoteExtension(sipDB* currentSipCall, char *newdstExtension){

         SIPLOG("UpdatingRemoteExtension from %s to %s\n",currentSipCall->remote_extension,newdstExtension);
         currentSipCall->remoteParty = 1;
         memset(currentSipCall->remote_extension,'\0',sizeof(currentSipCall->remote_extension));
         strncpy(currentSipCall->remote_extension,newdstExtension,sizeof(currentSipCall->remote_extension));
         int dst_directory = match_corporate_directory(newdstExtension);
         if(dst_directory < 0){
                /* Extension number not matched in corporate directory, may be this extension is not added in the
                   corporate directory by the admin */
         }
         else{
                memset(currentSipCall->dst_username,'\0',sizeof(currentSipCall->dst_username));
                memcpy(currentSipCall->dst_username,directory[dst_directory].name,sizeof(currentSipCall->dst_username));
        }
}

void updateMediaInfoForExistingCall(const char oldIp[], int oldAudioPort, int oldVideoPort, sipDB* currentSipCall, 
    const char newIp[], int newAudioPort, int newVideoPort)
{

    if(oldAudioPort) {
        SIPACCESS_LOCK;
        RTPACCESS_LOCK;
        deleteRTPDissector(oldAudioPort, oldIp);
        //deleteRTPCall(currentSipCall->myRTPCall);
        //currentSipCall->myRTPCall = createNewRTPCall(currentSipCall, 
        //    newAudioPort,
        //    newIp);
        currentSipCall->myRTPCall->port = newAudioPort;
        strncpy(currentSipCall->myRTPCall->ip_addr, newIp, MAX_ASCII_ADDR_LEN);        
        addRTPDissector(newAudioPort, newIp);
        RTPACCESS_UNLOCK;
        SIPACCESS_UNLOCK;
    }
    if(oldVideoPort) {
        if(currentSipCall->video_active == 1) {
            SIPACCESS_LOCK;
            RTPACCESS_LOCK;
            deleteRTPDissector(oldVideoPort, oldIp);
/*            populatevideoRTPCall(currentSipCall->myRTPCall,
                newVideoPort,
                currentSipCall->videoRTPMAP,
                currentSipCall->videoFMTP,
                currentSipCall->videoCodecType,
                currentSipCall->videoPayloadType);                        
 */
            currentSipCall->myRTPCall->vport = newVideoPort;
            addRTPDissector(newVideoPort, newIp);
            RTPACCESS_UNLOCK;
            SIPACCESS_UNLOCK;
       }
    }     
}
